#!/usr/bin/perl
#Monitor EMC Celerra for operational indicators    S.Teeter   Aug-2010
#Send results to Nagios using nsca
#Print operations are for debugging; formal output goes to $dat
#Perl programmers, please forgive my perl; not my native language

#(1) Set required variables; alter as needed for local environment
$start=time();
open (TMP, "date '+%F %H:%M:%S' |") || die "Failed on date: $!\n";
while ($line=<TMP>) { chomp($datetime=$line); }
close (TMP);
print "Starting execution of $0 v1.5 at $datetime\n";
$dir="/opt/nsca";       # location of components
$nas="celerra.cmd";	# file of commands to test
$dat="nsca_celerra.dat";# output file formatted for nsca
$msglen=200;            # max length of status information msg
$warn=85;               # yellow alert
$crit=95;               # red alert

#(2) Get hostname and remove qualifiers if present
open (TMP, "hostname |") || die "Failed on hostname: $!\n";
while ($line=<TMP>) { chomp($string=$line); }
close (TMP);
@parm=split(/\./,$string);
$host=$parm[0];
$host =~ tr/[A-Z]/[a-z]/;
print "Hostname $host\n";

#(3) Process file of commands
#    Format is service \t nas_command
$timer=time();
open (DAT, ">$dir/$dat") or die "Failed on open $dat: $!\n";
open (NAS,"< $dir/$nas") or die "Failed on open $nas: $!\n";
while ($line=<NAS>) {
    chomp($string=$line);
    if (substr($string,0,1) eq "#") {next;}     # skip comments
    @parm=split('	',$string);
    @result=&check_celerra($parm[1]);
    if ($parm[1] eq "nas_inventory -list")	{ &nas_inventory(); next; }
    if ($parm[1] eq "enclosure_status -v -e 0") { &enclosure_status(); next; }
    if ($parm[1] eq "nas_server -list")	{ &nas_server(); next; }
    if ($parm[1] eq "nas_fs -list")	{ &nas_fs(); next; }
    } # end while NAS
close (NAS);
close (DAT);
$timer=time()-$timer;
print "$0 took $timer seconds\n";

#(4) Subroutine to run Celerra command
sub check_celerra {
    @cmd_line=();               # array for Celerra command output
    print "Checking command $_[0]\n";
    open (TMP, "$_[0] 2>&1 |") || die "Failed on command $_[0]: $!\n";
    while ($line=<TMP>) { chomp($str=$line); push (@cmd_line, $str); }
    close(TMP);
    return @cmd_line;
    } # end subroutine

#(5) Subroutine to print formal and debugging outputs
sub print_results {
    if ($_[2] eq 3) { foreach (@result) { print "... $_\n"; } }
    $_[3]=~ s/ +/ /g; # remove extra spaces from msg
    if (length($_[3])>$msglen) {
        $_[3]=substr($_[3],0,$msglen-10) . " (more...)"; }
    printf "%s\t%s\t%d\t%s\n",$_[0],$_[1],$_[2],$_[3];
    printf DAT "%s\t%s\t%d\t%s\n",$_[0],$_[1],$_[2],$_[3];
    } # end subroutine

#(6) Subroutine to handle 'nas_inventory -list'
#    Sample command output:
#Component                                Type                 Status                    System ID
#Battery A                                Battery              OK                        CLARiiON CX4-240 FNM00091100078
#Celerra NS40G FNM000911000780000         Celerra              Warning                   Celerra NS40G FNM000911000780000  
#SP A                                     SP                   Storage processor is rea+ CLARiiON CX4-240 FNM00091100078    

#Status is OK, Warning, or Empty
sub nas_inventory {
#   Find column positions given variable word count, space delimited output
    $comp_ix=index($result[0],"Component");	
    $type_ix=index($result[0],"Type");	
    $stat_ix=index($result[0],"Status");	
    $syst_ix=index($result[0],"System ID");	
    $msg="";
    $rc=-1;
    $invct=0;
    foreach (@result) {
	if ($comp_ix eq -1) { last; }	# command output not as expected
        if (substr($_,$stat_ix,6) eq "Status") { $rc=0; next; }	# header
	if (length($_) < $stat_ix+6) { next; }		# blank line
	++$invct;
        if (substr($_,$stat_ix,2) eq "OK") { next; }
	if (substr($_,$stat_ix,6) eq "Empty.") { next; }
	if (substr($_,$stat_ix,7) eq "Warning" and $rc<2) { $rc=1; }
	else { $rc=2; }
	if ($msg ne "") { $msg = $msg . ", "; }
	$msg = $msg . substr($_,$comp_ix,($type_ix - $comp_ix)) . " status is " 
	    . substr($_,$stat_ix,($syst_ix - $stat_ix));
        } # end foreach
    if ($rc eq 0) {
        $msg="Status of $invct components is normal"; }
    elsif ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($host,$parm[0],$rc,$msg);
    } # end subroutine

#(7) Subroutine to handle 'enclosure_status -v -e 0'
#    Sample command output:
# DEVICE A                         DEVICE B
#---------------ENCLOSURE ALARMS----------------
# Pass        PSA OK               Pass
sub enclosure_status {
#   Find column positions given variable word count, space delimited output
    $deva_ix=index($result[1],"DEVICE A");	
    $devb_ix=index($result[1],"DEVICE B");	
    $msg="";
    $rc=-1;
    $skip=1;	# skip items other than alarms & errors
    foreach (@result) {
	if ($deva_ix eq -1) { last; }	# command output not as expected
	if (length($_) < $devb_ix+4) { next; }		# blank line
	if ($_ =~ m/------/) { 		# process block header
	    if ($rc eq -1) { $rc=0; }
	    if ($_ =~ /ALARM/) { $skip=0; next; }
	    if ($_ =~ /ERRORS/) { $skip=0; next; }
	    if ($_ =~ /FRU STATUS/) { $skip=0; next; }
	    $skip=1; next; }
	if ($skip eq 1) { next; }
        if (substr($_,$deva_ix,4) eq "Pass" 
	    and substr($_,$devb_ix,4) eq "Pass") { next; }
        $rc=2;
        if ($msg ne "") { $msg = $msg . ", "; }
	$msg = $msg . $_;
        } # end foreach
    if ($rc eq 0) {
        $msg="All alarms are in condition Pass"; }
    elsif ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($host,$parm[0],$rc,$msg);
    } # end subroutine

#(8) Subroutine to handle 'nas_server -list'
#    Sample command output:
#id      type  acl  slot groupID  state  name
#1        1    0     2              0    server_2
#2        4    0     3              0    server_3
sub nas_server {
@state=("enabled","disabled","failed over");
@type=("?", "nas", "?", "?","standby", "?", "rdf");
#   Parse command output and compute return code
    $msg="";
    $rc=-1;
    foreach (@result) {
	@val=split(' ',$_);
	if ($#val<0) { next; }			# blank line
	if ($val[0] eq "id") { $rc=0; next; }	# header
	$svr_name=$val[$#val];
	$svr_state=$state[$val[$#val-1]];
	$svr_type=$type[$val[1]];
	if ($msg ne "") { $msg = $msg . ", "; }
	$msg = $msg . $svr_name . " (" . $svr_type . ") state " . $svr_state;
	if ($svr_state eq "enabled") { next; }
	if ($svr_type eq "nas" and $svr_state eq "disabled") { $rc=2; next; }
	$rc=1;
	} # end foreach
    if ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($host,$parm[0],$rc,$msg);
    } # end subroutine

#(9) Subroutine to handle 'nas_fs -list'
#    Sample command output:
#nas_fs -list
#id      inuse type acl   volume    name                server
#1         n    1   0     10        root_fs_1           
#2         y    1   0     12        root_fs_2           1
#25        y    1   0     117       srv-sc-file01       v1
#nas_fs -size srv-sc-file01
#total = 3372838 avail = 1326502 used = 2046336 ( 61% ) (sizes in MB) ( blockcount = 7014608896 )
#volume: total = 3425102 (sizes in MB) ( blockcount = 7014608896 )
sub nas_fs {
#   Parse command output and compute return code
    $msg="";
    $rc=-1;
    $fsct=0;
    foreach (@result) {
        @val=split(' ',$_);
        if ($val[0] eq "id") { $rc=0; next; }   # header
        if ($#val<0) { next; }                  # blank line
	if ($val[1] ne "y") { next; }		# in-use only
	if ($val[2] ne "1" and $val[2] ne "5") { next; } # uxfs & rawfs only
	++$fsct;
#	Check filesystem utilization
        $fs_name=$val[5];
	$size_cmd="nas_fs -size " . $fs_name;
	@size=&check_celerra($size_cmd,"");
	foreach (@size) {
	    @val=split(' ',$_);
            if ($#val<0) { next; }           	# blank line
	    if ($val[0] ne "total") { next; }
	    $util=$val[8]/$val[2]*100;
	    $fsrc=($util > $crit) ? 2 : (($util > $warn) ? 1 : 0);
	    if ($fsrc eq 0) { next; }
	    $rc=($fsrc gt $rc ? $fsrc : $rc);
            if ($msg ne "") { $msg = $msg . ", "; }
            $msg = $msg . $fs_name . " is " . $val[10] . " full";
            } # end foreach @size
	} # end foreach @result
    if ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    elsif ($rc eq 0) {
        $msg="all are less than " . $warn . "% full"; }
    $msg="Of $fsct in-use filesystems- " . $msg;
    &print_results($host,$parm[0],$rc,$msg);
    } # end subroutine

