#!/usr/bin/perl
#Monitor EMC SP's for operational indicators	S.Teeter   Aug-2010
#Send results to Nagios using nsca
#Print operations are for debugging; formal output goes to $dat
#See http://www.emc.com/microsites/clariion-support/pdf/300-003-628.pdf
#Perl programmers, please forgive my perl; not my native language

#(1) Set required variables; alter as needed for local environment
$start=time();
open (TMP, "date '+%F %H:%M:%S' |") || die "Failed on date: $!\n";
while ($line=<TMP>) { chomp($datetime=$line); }
close (TMP);
print "Starting execution of $0 v1.4 at $datetime\n";
$dir="/opt/nsca/EMC";	# location of components
$emc="clariion.cmd";	# file of commands and SPs to test
$dat="nsca_clariion.dat"; # output file formatted for nsca
$msglen=200;            # max length of status information msg
$timeout=20;		# number of seconds to wait for naviseccli response
$retries=5;		# number of times to retry naviseccli command
$sleep=10;		# number of seconds to sleep between retries
$warn=85;               # yellow alert
$crit=95;               # red alert

#(2) Process file of commands and SPs
#    Format is service \t command \t sp-name
#    sp-name is fully qualified, so use spabbr for Nagios nsca
$timer=time();
open (DAT, ">$dir/$dat") or die "Failed on open $dat: $!\n";
open (EMC,"< $dir/$emc") or die "Failed on open $emc: $!\n";
while ($line=<EMC>) {
    chomp($string=$line);
    if (substr($string,0,1) eq "#") { next; }	# skip comments
    @parm=split('	',$string);
    @result=&check_sp($parm[2],$parm[1]);
    @parm2=split(/\./,$parm[2]);
    $spabbr=lc $parm2[0];
    if ($parm[1] eq "faults -list")		{ &faults_list(); next; }
    if ($parm[1] eq "getall -cache")		{ &getall_cache(); next; }
    if ($parm[1] eq "getrg -type -state")	{ &getrg_state(); next; }
    if ($parm[1] eq "getlun -type -state")	{ &getlun_state(); next; }
    } # end while EMC
close (EMC);
close (DAT);
$timer=time()-$timer;
print "$0 took $timer seconds\n";

#(3) Subroutine to run CLI command
#    Deal with socket errors that occur sometimes; message looks like this:
#Error occurred during response from the target
#Message :  Socket stream invalid
#Error occurred during HTTP request/response from the target: '145.36.241.145'.
#Message : End of data stream
sub check_sp {
    $naviseccli="naviseccli -timeout $timeout -Address ";
    $ctr=1;
    while ($ctr<=$retries) {	# attempt command $retries times
	$err=0;
	@cli_line=();		# array for CLI command output
	print "Checking SP $_[0] with $_[1]\n";
	open (TMP, "$naviseccli $_[0] $_[1] 2>&1 |") || die "Failed on navi: $!\n";
	while ($line=<TMP>) {
	    chomp($string=$line);
	    if ($string =~ m/Socket stream invalid/) { $err=1; }
	    push (@cli_line, $string); }
	close(TMP);
	if ($err==0) { return @cli_line; }
	print "... socket error on attempt $ctr\n";
	sleep $sleep;
	++$ctr;
	} # end while $ctr
    } # end subroutine

#(4) Subroutine to print formal and debugging outputs
sub print_results {
    if ($_[2] eq 3) { foreach (@result) { print "... $_\n"; } }
    $_[3]=~ s/ +/ /g; # remove extra spaces from msg
    if (length($_[3])>$msglen) {
	$_[3]=substr($_[3],0,$msglen-10) . " (more...)"; }
    printf "%s\t%s\t%d\t%s\n",$_[0],$_[1],$_[2],$_[3];
    printf DAT "%s\t%s\t%d\t%s\n",$_[0],$_[1],$_[2],$_[3];
    } # end subroutine

#(5) Subroutine to handle 'faults -list'
#    Sample command output:
#The array is operating normally.
sub faults_list {
    $msg="";
    $rc=-1;
    foreach (@result) {
	if ($_ =~ m/normally/) {
	    $rc=0; $msg = $_; last; }			# no faults
	if ($_ =~ m/Faulted/ or $_ =~ m/Removed/) {
            if ($msg ne "") { $msg=$msg . ", "; }
            $msg = $msg . $_;
	    if ($_ =~ m/Faulted/) { $rc=2; next; }      # alert
            $rc=($rc>1) ? $rc : 1;                      # warning
            } # end if Faulted or Removed
	} # end foreach
    if ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($spabbr,$parm[0],$rc,$msg);
    } # end subroutine

#(6) Subroutine to handle 'getall -cache'
#    Sample command output:
#SPA Read Cache State                Enabled
#SPB Read Cache State                Enabled
#SPA Write Cache State               Enabled
#SPB Write Cache State               Enabled
sub getall_cache{
    $msg="";
    $rc=-1;
    foreach (@result) {
	if ($_ =~ m/Cache State/) {
	    if ($_ =~ m/Enabled/) {
		$rc=($rc>0) ? $rc : 0; next; }		# state is OK
	    if ($msg ne "") { $msg=$msg . ", "; }
	    $msg = $msg . $_;
	    if ($_ =~ m/Disabled/) { $rc=2; next; }	# alert
	    $rc=($rc>1) ? $rc : 1;			# warning
	    } # end if Cache State
        } # end foreach
    if ($rc==0) { 
	$msg="SPs Read and Write Cache State Enabled"; }
    elsif ($rc<0) { 
	$rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($spabbr,$parm[0],$rc,$msg);
    } # end subroutine

#(7) Subroutine to handle 'getrg -state -type'
#    Sample command output:
#RaidGroup ID:                              0
#RaidGroup Type:                            r5
#RaidGroup State:                           Explicit_Remove
#                                           Valid_luns
sub getrg_state{
    $rg=-1;
    $rgct=0;
    $type="";
    $state="";
    $msg="";
    $rc=-1;
    foreach (@result) {
	@word=split(' ',$_);
	if ($#word<0) { next; }			# blank line
	if ($word[1] eq "ID:") { $rg=$word[2]; 
            $rc=($rc>0) ? $rc : 0; next; }
	if ($word[1] eq "Type:") { $type=$word[2]; next; }
	if ($type eq "hot_spare") { next; }	# ignore hot spares
	++$rgct;
	if ($word[1] eq "State:") { $state=$word[2]; }
	    elsif ($#word==0) { $state=$word[0]; }
	    else { next; }
	if ($state eq "Explicit_Remove" 
	    or $state eq "Valid_luns") { next; }	# state OK
	if ($msg ne "") { $msg=$msg . ", "; }
	$msg = $msg . "RAID group $rg (type $type) is $state"; next;
        $rc=($rc>1) ? $rc : 1; 			# at least a warning
	if ($state eq "Invalid" or $state eq "Halted") { $rc=2; } # alert
        } # end foreach
    if ($rc==0) {
        $msg="States of $rgct RAID groups are valid"; }
    elsif ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($spabbr,$parm[0],$rc,$msg);
    } # end subroutine

#(8) Subroutine to handle 'getlun -type -state'
#    Sample command output:
#LOGICAL UNIT NUMBER 626
#RAID Type:                  N/A
#State:                      Bound
sub getlun_state{
    $lun=-1;
    $lunct=0;
    $type="";
    $state="";
    $msg="";
    $rc=-1;
    foreach (@result) {
        @word=split(' ',$_);
        if ($#word<0) { next; }                 # blank line
        if ($word[0] eq "LOGICAL") { $lun=$word[3];
            $rc=($rc>0) ? $rc : 0; next; }
        if ($word[1] eq "Type:") { $type=$word[2]; next; }
        if ($type eq "Hot") { next; }     # ignore hot spares
        ++$lunct;
        if ($word[0] eq "State:") { $state=$word[1]; }
            else { next; }
        if ($state eq "Bound") { next; }		# state OK
        if ($msg ne "") { $msg=$msg . ", "; }
        $msg = $msg . "LUN $lun (type $type) is $state"; next;
        $rc=($rc>1) ? $rc : 1;                  # at least a warning
        if ($state eq "Faulted") { $rc=2; } 	# alert
        } # end foreach
    if ($rc==0) {
        $msg="States of $lunct LUNs are valid"; }
    elsif ($rc<0) {
        $rc=3; $msg="Unexpected results from command " . $parm[1]; }
    &print_results($spabbr,$parm[0],$rc,$msg);
    } # end subroutine
